﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace WorldView
{
    public partial class TerrariaCreate : Form
    {
        private WorldMapper mapper = null;
        private Varies v;
        private string ChestPath = "";
        private string SignPath = "";
        private string NpcPath = "";
        private string WallPath = "";
        private bool WorldG = false;
        private bool WorldG2 = false;

        public TerrariaCreate()
        {
            mapper = new WorldMapper();
            mapper.Initialize();
            v = new Varies();
            v.init();
            InitializeComponent();
            checkBoxUseSymbols.Enabled = false;
            checkBoxDrawWalls.Enabled = false;
            labelSpecialThanks.Text = Constants.Credits;
        }

        private void WorldViewForm_Load(object sender, EventArgs e)
        {

        }

        private void WorldViewForm_OpenWorldClicked(object sender, EventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = "Terraria World (*.wld)|*.wld|Terraria Backup World (*.wld.bak)|*.wld.bak";
            dialog.Title = "Select World File";
            string filePath = (dialog.ShowDialog() == DialogResult.OK) ? dialog.FileName : string.Empty;

            if (filePath == string.Empty)
            {
                return;
            }
            this.worldFilePathTextBox.Text = filePath;

            try
            {
                mapper.OpenWorld(filePath);
                PopulateTree();
                outputFileBrowseButton.Enabled = true;
                outputFileConfirmButton.Enabled = false;
                checkBoxUseSymbols.Enabled = true;
                checkBoxDrawWalls.Enabled = true;
                ChestCheck.Enabled = true;
                checkBox2.Enabled = true;
                checkBox3.Enabled = true;
                outputFileConfirmButton.Text = "Create";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }



        private void PopulateTree()
        {
            worldPropertyGrid.SelectedObject = mapper.Header;
        }

        private void outputFileBrowseButton_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = "PNG (*.png)|*.png";
            dialog.Title = "Select World File";
            dialog.FileName = string.Format("{0}.png", System.IO.Path.GetFileNameWithoutExtension(this.worldFilePathTextBox.Text));
            string filePath = (dialog.ShowDialog() == DialogResult.OK) ? dialog.FileName : string.Empty;
            if (filePath == string.Empty)
            {
                return;
            }
            else if (new System.IO.FileInfo(filePath).Extension.ToUpper() != ".PNG")
            {
                MessageBox.Show("Output File MUST End in .png", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            outputFileTextBox.Text = filePath;
            outputFileConfirmButton.Enabled = true;
        }

        private void outputFileConfirmButton_Click(object sender, EventArgs e)
        {
            try
            {
                outputFileConfirmButton.Enabled = false;
                outputFileConfirmButton.Text = "Writing Image File...";
                mapper.CreatePreviewPNG(outputFileTextBox.Text, checkBoxDrawWalls.Checked, checkBoxUseSymbols.Checked, progressBarOutputPreview,ChestPath,SignPath,NpcPath,WallPath,World,Wall);
                outputFileConfirmButton.Text = "File Written!";
                outputFileBrowseButton.Enabled = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            mapper.CloseWorld();
            outputFileTextBox.Text = string.Empty;
            worldFilePathTextBox.Text = string.Empty;
        }



        private void WorldIButton_Click_1(object sender, EventArgs e)
        {
            OpenFileDialog dialogI = new OpenFileDialog();
            dialogI.Filter = "World Image File (*.png)|*.png";
            dialogI.Title = "Select World Image File";
            string filePath = (dialogI.ShowDialog() == DialogResult.OK) ? dialogI.FileName : string.Empty;

            if (filePath == string.Empty)
            {
                return;
            }
            this.WorldIText.Text = filePath;
            button3.Enabled = true;
            
        }

        private void WorldOButton_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialogW = new SaveFileDialog();
            dialogW.Filter = "Terraria World File (*.wld)|*.wld";
            dialogW.Title = "Select World File";
            dialogW.FileName = string.Format("{0}.wld", System.IO.Path.GetFileNameWithoutExtension(this.WorldOText.Text));
            string filePath = (dialogW.ShowDialog() == DialogResult.OK) ? dialogW.FileName : string.Empty;
            if (filePath == string.Empty)
            {
                return;
            }
            else if (new System.IO.FileInfo(filePath).Extension.ToUpper() != ".WLD")
            {
                MessageBox.Show("Output File MUST End in .wld", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            WorldOText.Text = filePath;
            WorldOExport.Enabled = true;
            
        }

        private void WorldOExport_Click(object sender, EventArgs e)
        {
            if (Convert.ToInt32(SpawnXBox.Text) < 1 && Convert.ToInt32(SpawnYBox.Text) < 3 )
            {
                MessageBox.Show("Change the spawn position, otherwise Terraria will crash on load.\n (If you used the viewer, just put in random numbers and the spawn will be normal)");
            }
            else
            {
                Worker1.RunWorkerAsync();
            }
         }

        private void button1_Click(object sender, EventArgs e)
        {
            mapper.CreateReference();
        }

        private void ChestButton_Click(object sender, EventArgs e)
        {
            ChestWindow ChestWin = new ChestWindow(v);
            ChestWin.Show();
            MessageBox.Show("When typing in the XY coordinates for a chest, always use the top left pixel of the chest.", "XY coordinates", MessageBoxButtons.OK, MessageBoxIcon.Information);
           // button2.Enabled = true;
        }

        private void NpcButton_Click(object sender, EventArgs e)
        {
            NpcWindow NpcWin = new NpcWindow(v);
            NpcWin.Show();
           
        }

        private void button2_Click(object sender, EventArgs e)
        {
            SignWindow SignWin = new SignWindow(v);
            SignWin.Show();
            MessageBox.Show("When typing in the XY coordinates for a sign, always use the top left pixel of the sign.", "XY coordinates", MessageBoxButtons.OK, MessageBoxIcon.Information);

            //NpcButton.Enabled = true;
        }

        private void ChestCheck_CheckedChanged(object sender, EventArgs e)
        {
            if (ChestCheck.Checked)
            {
                SaveFileDialog dialogW = new SaveFileDialog();
                dialogW.Filter = "Terraria Chest File (*.chest)|*.chest";
                dialogW.Title = "Select Chest File";
                dialogW.FileName = string.Format("{0}-Chest.chest", System.IO.Path.GetFileNameWithoutExtension(this.worldFilePathTextBox.Text));
                string filePath = (dialogW.ShowDialog() == DialogResult.OK) ? dialogW.FileName : string.Empty;
                if (filePath == string.Empty)
                {
                    return;
                }
                else if (new System.IO.FileInfo(filePath).Extension.ToUpper() != ".CHEST")
                {
                    MessageBox.Show("Output File MUST End in .chest", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                this.ChestPath = filePath;
            }
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox2.Checked)
            {
                SaveFileDialog dialogW = new SaveFileDialog();
                dialogW.Filter = "Terraria Sign File (*.sign)|*.sign";
                dialogW.Title = "Select Sign File";
                dialogW.FileName = string.Format("{0}-Sign.sign", System.IO.Path.GetFileNameWithoutExtension(this.worldFilePathTextBox.Text));
                string filePath = (dialogW.ShowDialog() == DialogResult.OK) ? dialogW.FileName : string.Empty;
                if (filePath == string.Empty)
                {
                    return;
                }
                else if (new System.IO.FileInfo(filePath).Extension.ToUpper() != ".SIGN")
                {
                    MessageBox.Show("Output File MUST End in .SIGN", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                this.SignPath = filePath;
            }
        }

        private void checkBox3_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox3.Checked)
            {
                SaveFileDialog dialogW = new SaveFileDialog();
                dialogW.Filter = "Terraria Npc File (*.npc)|*.npc";
                dialogW.Title = "Select Npc File";
                dialogW.FileName = string.Format("{0}-NPC.npc", System.IO.Path.GetFileNameWithoutExtension(this.worldFilePathTextBox.Text));
                string filePath = (dialogW.ShowDialog() == DialogResult.OK) ? dialogW.FileName : string.Empty;
                if (filePath == string.Empty)
                {
                    return;
                }
                else if (new System.IO.FileInfo(filePath).Extension.ToUpper() != ".NPC")
                {
                    MessageBox.Show("Output File MUST End in .npc", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                this.NpcPath = filePath;
            }
        }

        private void checkBoxDrawWalls_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxDrawWalls.Checked)
            {
                SaveFileDialog dialogW = new SaveFileDialog();
                dialogW.Filter = "Terraria Wall File (*.png)|*.png";
                dialogW.Title = "Select Wall File";
                dialogW.FileName = string.Format("{0}-Wall.png", System.IO.Path.GetFileNameWithoutExtension(this.worldFilePathTextBox.Text));
                string filePath = (dialogW.ShowDialog() == DialogResult.OK) ? dialogW.FileName : string.Empty;
                if (filePath == string.Empty)
                {
                    return;
                }
                else if (new System.IO.FileInfo(filePath).Extension.ToUpper() != ".PNG")
                {
                    MessageBox.Show("Output File MUST End in .png", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                this.WallPath = filePath;
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            OpenFileDialog dialogI = new OpenFileDialog();
            dialogI.Filter = "World Wall Image File (*.png)|*.png";
            dialogI.Title = "Select World Image File";
            string filePath = (dialogI.ShowDialog() == DialogResult.OK) ? dialogI.FileName : string.Empty;

            if (filePath == string.Empty)
            {
                return;
            }
            this.WorldWOut.Text = filePath;
            try
            {
                WorldOButton.Enabled = true;
                outputFileConfirmButton.Enabled = false;
                outputFileConfirmButton.Text = "Create";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }

        private void ReleaseBox_MaskInputRejected(object sender, MaskInputRejectedEventArgs e)
        {

        }

        private void Worker1_DoWork(object sender, DoWorkEventArgs e)
        {
            Random r = new Random();
            mapper.AnalyseWorld(WorldBox.Text, Convert.ToInt32(ReleaseBox.Text), r.Next(10000), Convert.ToInt32(SpawnXBox.Text), Convert.ToInt32(SpawnYBox.Text), WorldOText.Text, CthuluBox.Checked, DevourerBox.Checked, SkeletronBox.Checked, Worker1, WorldIText.Text, Convert.ToInt32(Slevel.Text), Convert.ToInt32(Rlevel.Text), v, WorldWOut.Text);
        }

        private void Worker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            WorldOfProgress.Value = e.ProgressPercentage;
        }


    }
}