﻿using System.Drawing;
using System.IO;

namespace WorldView
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Drawing.Imaging;

    public class WorldReader
    {
        private long tileOffset;
        private long chestOffset;

        private string filePath;
        private FileStream stream;
        private BinaryReader reader;

        public WorldReader(string filePath)
        {
            this.filePath = filePath;
            this.stream = new FileStream(this.filePath, FileMode.Open);
            this.reader = new BinaryReader(stream);
            tileOffset = 0;
            chestOffset = 0;
        }

        public void SeekToTiles()
        {
            this.stream.Seek(this.tileOffset, SeekOrigin.Begin);
        }

        public WorldHeader ReadHeader()
        {
            WorldHeader header;
            this.stream.Seek(0, SeekOrigin.Begin);
            int num = this.reader.ReadInt32();
            if (num == 0x26)
            {
                WorldHeader header2 = new WorldHeader
                {
                    ReleaseNumber = num,
                    Name = this.reader.ReadString(),
                    WorldCoords = new Rect(this.reader.ReadInt32(), this.reader.ReadInt32(), this.reader.ReadInt32(), this.reader.ReadInt32()),
                    MaxTiles = new Point(this.reader.ReadInt32(), this.reader.ReadInt32()),
                    SpawnPoint = new Point(this.reader.ReadInt32(), this.reader.ReadInt32()),
                    SurfaceLevel = this.reader.ReadDouble(),
                    RockLayer = this.reader.ReadDouble(),
                    TemporaryTime = this.reader.ReadDouble(),
                    IsDayTime = this.reader.ReadBoolean(),
                    MoonPhase = this.reader.ReadInt32(),
                    IsBloodMoon = this.reader.ReadBoolean(),
                    DungeonPoint = new Point(this.reader.ReadInt32(), this.reader.ReadInt32()),
                    IsBoss1Dead = this.reader.ReadBoolean(),
                    IsBoss2Dead = this.reader.ReadBoolean(),
                    IsBoss3Dead = this.reader.ReadBoolean(),
                    IsShadowOrbSmashed = this.reader.ReadBoolean(),
                    IsMeteorSpawned = this.reader.ReadBoolean(),
                    ShadowOrbsSmashed = this.reader.ReadByte(),
                    InvasionDelay = this.reader.ReadInt32(),
                    InvasionSize = this.reader.ReadInt32(),
                    InvasionType = this.reader.ReadInt32(),
                    InvasionPointX = this.reader.ReadDouble()
                };
                header = header2;
            }
            else
            {
                WorldHeader header3 = new WorldHeader
                {
                    ReleaseNumber = num,
                    Name = this.reader.ReadString(),
                    Id = this.reader.ReadInt32(),
                    WorldCoords = new Rect(this.reader.ReadInt32(), this.reader.ReadInt32(), this.reader.ReadInt32(), this.reader.ReadInt32()),
                    MaxTiles = new Point(this.reader.ReadInt32(), this.reader.ReadInt32()),
                    SpawnPoint = new Point(this.reader.ReadInt32(), this.reader.ReadInt32()),
                    SurfaceLevel = this.reader.ReadDouble(),
                    RockLayer = this.reader.ReadDouble(),
                    TemporaryTime = this.reader.ReadDouble(),
                    IsDayTime = this.reader.ReadBoolean(),
                    MoonPhase = this.reader.ReadInt32(),
                    IsBloodMoon = this.reader.ReadBoolean(),
                    DungeonPoint = new Point(this.reader.ReadInt32(), this.reader.ReadInt32()),
                    IsBoss1Dead = this.reader.ReadBoolean(),
                    IsBoss2Dead = this.reader.ReadBoolean(),
                    IsBoss3Dead = this.reader.ReadBoolean(),
                    IsShadowOrbSmashed = this.reader.ReadBoolean(),
                    IsMeteorSpawned = this.reader.ReadBoolean(),
                    ShadowOrbsSmashed = this.reader.ReadByte(),
                    InvasionDelay = this.reader.ReadInt32(),
                    InvasionSize = this.reader.ReadInt32(),
                    InvasionType = this.reader.ReadInt32(),
                    InvasionPointX = this.reader.ReadDouble()
                };
                header = header3;
            }
            this.tileOffset = this.stream.Position;
            return header;
        }

        public TileTypeW GetNextTile()
        {
            bool isTileActive = reader.ReadBoolean();
            TileTypeW tileType = new TileTypeW(false,TileTypeW.TileType.Sky, TileTypeW.TileType.Sky);
            byte blockType = 0x00;
            if (isTileActive)
            {
                blockType = reader.ReadByte();
                if (WorldMapper.TileTypeWDefs[blockType].IsImportant)
                {
                    reader.ReadInt16();
                    reader.ReadInt16();
                }
                tileType.t = WorldMapper.TileTypeWDefs[blockType].TileType;
            }
            else
            {
                tileType.t = TileTypeW.TileType.Sky;
            }
            bool isLighted = reader.ReadBoolean();
            bool isWall = reader.ReadBoolean();
            tileType.wall = isWall;
            if (isWall)
            {
                byte wallType = reader.ReadByte();
                tileType.Wallt = WorldMapper.TileTypeWDefs[(int)wallType + Constants.WallOffset].TileType;
            }
            bool isLiquid = reader.ReadBoolean();
            if (isLiquid)
            {
                byte liquidLevel = reader.ReadByte();
                bool isLava = reader.ReadBoolean();
                if (isWall || tileType.t == TileTypeW.TileType.Sky)
                {
                    tileType.t = isLava ? TileTypeW.TileType.Lava : TileTypeW.TileType.Water;
                }
                
            }
            return tileType;
        }

        /// <summary>
        /// Gets the next chest, if the next item is not a chest, return null
        /// </summary>
        /// <returns></returns>

        public void GetRest(string CFile, string SFile, string NFile)
        {
            if (CFile != "")
            {
                using (FileStream stream1 = new FileStream(CFile, FileMode.OpenOrCreate))
                {
                    using (BinaryWriter writer = new BinaryWriter(stream1))
                    {
                        for (int i = 0; i < 1000; i++)
                        {
                            bool active = reader.ReadBoolean();
                            writer.Write(active);
                            if (active)
                            {
                                writer.Write(reader.ReadInt32());
                                writer.Write(reader.ReadInt32());
                                for (int k = 0; k < 20; k++)
                                {
                                    int count = reader.ReadByte();
                                    writer.Write(count);
                                    if (count > 0)
                                    {
                                        writer.Write(reader.ReadString());
                                    }
                                }
                            }
                        }
                        writer.Close();
                    }

                }
            }
            else
            {
                for (int i = 0; i < 1000; i++)
                {
                    if (reader.ReadBoolean())
                    {
                        reader.ReadInt32();
                        reader.ReadInt32();
                        for (int k = 0; k < 20; k++)
                        {
                            if(reader.ReadByte() > 0)
                            {
                                reader.ReadString();
                            }
                        }
                    }
                }
            }
            if (SFile != "")
            {
                using (FileStream stream2 = new FileStream(SFile, FileMode.OpenOrCreate))
                {
                    using (BinaryWriter writer = new BinaryWriter(stream2))
                    {

                        for (int i = 0; i < 1000; i++)
                        {
                            bool active = reader.ReadBoolean();
                            writer.Write(active);
                            string text = "";
                            int X = 0;
                            int Y = 0;
                            if (active)
                            {
                                text = reader.ReadString();
                                X = reader.ReadInt32();
                                Y = reader.ReadInt32();

                                writer.Write(text);
                                writer.Write(X);
                                writer.Write(Y);
                            }
                        }
                        writer.Close();
                    }

                }
            }
            else
            {
                for (int i = 0; i < 1000; i++)
                {
                    if (reader.ReadBoolean())
                    {
                        reader.ReadString();
                        reader.ReadInt32();
                        reader.ReadInt32();
                    }
                }

            }
            if (NFile != "")
            {
                using (FileStream stream3 = new FileStream(NFile, FileMode.OpenOrCreate))
                {
                    using (BinaryWriter writer = new BinaryWriter(stream3))
                    {

                        bool active = reader.ReadBoolean();
                        int i = 0;
                        writer.Write(active);

                        while (active)
                        {
                            writer.Write(reader.ReadString());
                            writer.Write(reader.ReadSingle());
                            writer.Write(reader.ReadSingle());
                            writer.Write(reader.ReadBoolean());
                            writer.Write(reader.ReadInt32());
                            writer.Write(reader.ReadInt32());
                            try
                            {
                                active = reader.ReadBoolean();
                                writer.Write(active);
                            }
                            catch (EndOfStreamException)
                            {
                                active = false;
                                writer.Write(false);
                            }
                            i++;


                            
                        }
                        writer.Close();
                    }


                }
            }

            }
        

        public void Close()
        {
            this.reader.Close();
        }

        //public void CreatePreviewImage(string outputFilePath)
        //{
        //    int maxX = (int)Header.MaxTiles.Y;
        //    int maxY = (int)Header.MaxTiles.X;
        //    //byte[,] pixels = new byte[cols, rows];

        //    Bitmap bitmap = new Bitmap(maxX, maxY);
        //    Graphics g2 = Graphics.FromImage((Image)bitmap);
        //    g2.FillRectangle(Brushes.SkyBlue, 0, 0, bitmap.Width, bitmap.Height);
        //    Dictionary<byte, Color> randomColors = new Dictionary<byte, Color>();
        //    Random random = new Random();
        //    for (int col = 0; col < maxX; col++)
        //    {
        //        for (int row = 0; row < maxY; row++)
        //        {
        //            bool isTileActive = reader.ReadBoolean();
        //            TileTypeW TileTypeW = TileTypeW.TileType.Unknown;
        //            byte blockType = 0x00;
        //            if (isTileActive)
        //            {
        //                blockType = reader.ReadByte();
        //                if (TileTypeWDefs[blockType].isImportant)
        //                {
        //                    reader.ReadInt16();
        //                    reader.ReadInt16();
        //                }
        //                TileTypeW = TileTypeWDefs[blockType].TileTypeW;
        //            }
        //            else
        //            {
        //                TileTypeW = TileTypeW.TileType.Sky;
        //            }
        //            bool isLighted = reader.ReadBoolean();
        //            bool isWall = reader.ReadBoolean();
        //            if (isWall)
        //            {
        //                byte wallType = reader.ReadByte();
        //                if (TileTypeW == TileTypeW.TileType.Unknown)
        //                {
        //                    TileTypeW = TileTypeW.TileType.Wall;    
        //                }

        //               // bitmap.SetPixel(col, row, Color.PowderBlue);
        //            }
        //            bool isLiquid = reader.ReadBoolean();

        //            if (isLiquid)
        //            {
        //                byte liquidLevel = reader.ReadByte();
        //                bool isLava = reader.ReadBoolean();
        //                TileTypeW = isLava ? TileTypeW.TileType.Lava : TileTypeW.TileType.Water;
        //               // bitmap.SetPixel(col, row, isLava ? Color.Pink : Color.Blue);
        //            }
        //            //if (TileTypeW != TileTypeW.TileType.Lava || TileTypeW != TileTypeW.TileType.Wall || TileTypeW != TileTypeW.TileType.Water)
        //            //{
        //            //    if (!randomColors.ContainsKey(blockType))
        //            //    {

        //            //        int r = random.Next(0, 255);
        //            //        int g = random.Next(0, 255);
        //            //        int b = random.Next(0, 255);

        //            //        randomColors.Add(blockType, Color.FromArgb(r, g, b));
        //            //    }
        //            //    bitmap.SetPixel(col, row, randomColors[blockType]);
        //            //}
        //            //if (TileTypeW == TileTypeW.TileType.Unknown)
        //            //{
        //            //    if (!randomColors.ContainsKey(blockType))
        //            //    {
        //            //        int r = random.Next(0, 255);
        //            //        int g = random.Next(0, 255);
        //            //        int b = random.Next(0, 255);
        //            //        randomColors.Add(blockType, Color.FromArgb(r, g, b));
        //            //    }
        //            //    bitmap.SetPixel(col, row, randomColors[blockType]);
        //            //}
        //            if (TileTypeW != TileTypeW.TileType.Sky)
        //            {
        //                bitmap.SetPixel(col, row, tileColorDefs[TileTypeW]);
        //            }
        //            else
        //            {
        //                // Don't draw its sky
        //            }
        //        }
        //    }


        //    //TextWriter fs = new StreamWriter(@"C:\Users\Frank\Documents\My Games\Terraria\Worlds\dump.txt");
        //    //foreach (var pair in randomColors)
        //    //{
        //    //    fs.WriteLine(string.Format("{0}, ({1}, {2}, {3})", pair.Key, pair.Value.R, pair.Value.G, pair.Value.B));
        //    ////}
        //    //fs.Close();


        //    bitmap.Save(outputFilePath, ImageFormat.Png);
        //    reader.Close();
        //}
    }
}